package com.jianshui.api.controller.http.invoice.v1;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.dxhy.order.utils.Base64Util;
import com.jianshui.common.core.domain.AjaxResult;
import com.jianshui.common.core.domain.entity.Companyservice;
import com.jianshui.common.core.domain.entity.ele.DdfpzxxV5;
import com.jianshui.common.core.domain.entity.ele.DxhyInterfaceResponse;
import com.jianshui.common.core.domain.entity.ele.ResponseBO;
import com.jianshui.common.enums.ErrorCode;
import com.jianshui.common.utils.StringUtils;
import com.jianshui.invoice.service.ElephantInvoiceService;
import com.jianshui.system.service.ICompanyserviceService;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Slf4j
@RestController
@RequestMapping("/api/invoice/front/v1")
public class InvoiceFrontController {

    @Autowired
    private ICompanyserviceService iCompanyserviceService;

    @Autowired
    private ElephantInvoiceService elephantInvoiceService;


    /**
     * 数电登录接口(前端使用)
     */
    @ApiImplicitParams({
            @ApiImplicitParam(name = "identity", value = "身份认证", dataType = "java.lang.String", example = "1130", required = true),
            @ApiImplicitParam(name = "secret", value = "身份认证Key", dataType = "java.lang.String", example = "LTEO+oOgWMsuQAOUglqXuQ==", required = true),
            @ApiImplicitParam(name = "params", value = "请求体", dataType = "java.lang.String", required = true)
    })
    @PostMapping("/rpaLogin")
    public Object rpaLogin(@RequestBody Map<String,Object> params) {

        JSONObject paramJson = new JSONObject(params);

        log.debug("rpaLogin param{}", paramJson.toJSONString());

        //参数校验
        if (StringUtils.isBlank(paramJson.getString("identity"))){
            return AjaxResult.error("公司ID不能为空");
        }

        if (StringUtils.isBlank(paramJson.getString("secret"))){
            return AjaxResult.error("公司Key不能为空");
        }


        if (StringUtils.isBlank(paramJson.getString("yhm"))){
            return AjaxResult.error("电子税局登录名不能为空");
        }

        if (StringUtils.isBlank(paramJson.getString("mm"))){
            return AjaxResult.error("电子税局密码不能为空");
        }
        //方法名处理
        if (StringUtils.isBlank(paramJson.getString("ffm"))){
            return AjaxResult.error("方法名不能为空");
        }

        if ("0".equals(paramJson.getString("ffm"))){
            paramJson.put("ffm","login");
        }
        if ("1".equals(paramJson.getString("ffm"))){
            paramJson.put("ffm","send_sms");
        }
        if ("2".equals(paramJson.getString("ffm"))){
            if (StringUtils.isBlank(paramJson.getString("zrrlx"))){
                return AjaxResult.error("责任人类型不能为空");
            }
            paramJson.put("ffm","select_zrrlx");
        }
        if ("3".equals(paramJson.getString("ffm"))){
            if (StringUtils.isBlank(paramJson.getString("dxyzm"))){
                return AjaxResult.error("短信验证码不能为空");
            }
            paramJson.put("ffm","sms_login");
        }

        Companyservice companyservice = iCompanyserviceService.selectCompanyserviceByIdentity(paramJson.getString("identity"));
        if (companyservice == null || !paramJson.getString("secret").equals(companyservice.getSecret())) {
            return AjaxResult.error(ErrorCode.COMPANY_NOT_FOUND);
        }

        return elephantInvoiceService.rpaLogin(paramJson,companyservice);

    }


    /**
     * 数电账号查询接口(前端使用)
     */
    @ApiImplicitParams({
            @ApiImplicitParam(name = "identity", value = "身份认证", dataType = "java.lang.String", example = "1130", required = true),
            @ApiImplicitParam(name = "secret", value = "身份认证Key", dataType = "java.lang.String", example = "LTEO+oOgWMsuQAOUglqXuQ==", required = true)
    })
    @PostMapping("/queryRpaTaxAccount")
    public Object queryRpaTaxAccount(@RequestBody Map<String,Object> params){

        JSONObject paramJson = new JSONObject(params);

       String identity = paramJson.getString("identity");
       String secret = paramJson.getString("secret");
        log.debug("queryRpaTaxAccount param identity:{} secret:{}",identity,secret);
        //参数校验
        if (StringUtils.isBlank(identity)){
            return AjaxResult.error("公司ID不能为空");
        }

        if (StringUtils.isBlank(secret)){
            return AjaxResult.error("公司Key不能为空");
        }

        Companyservice companyservice = iCompanyserviceService.selectCompanyserviceByIdentity(identity);
        if (companyservice == null || !secret.equals(companyservice.getSecret())) {
            return AjaxResult.error(ErrorCode.COMPANY_NOT_FOUND);
        }

        return elephantInvoiceService.queryRpaTaxAccount(companyservice.getSellertax(),companyservice);

    }

    /**
     * 数电获取二维码接口(前端使用)
     */
    @ApiImplicitParams({
            @ApiImplicitParam(name = "identity", value = "身份认证", dataType = "java.lang.String", example = "1130", required = true),
            @ApiImplicitParam(name = "secret", value = "身份认证Key", dataType = "java.lang.String", example = "LTEO+oOgWMsuQAOUglqXuQ==", required = true),
            @ApiImplicitParam(name = "yhm", value = "电子税局登陆账号", dataType = "java.lang.String", required = true)
    })
    @PostMapping("/getRpaQrCode")
    public Object getRpaQrCode(@RequestBody Map<String,Object> params) {

        JSONObject paramJson = new JSONObject(params);
        log.debug("getRpaQrCode param params:{}", paramJson);

        String identity = paramJson.getString("identity");
        String secret = paramJson.getString("secret");
        //参数校验
        if (StringUtils.isBlank(identity)){
            return AjaxResult.error("公司ID不能为空");
        }

        if (StringUtils.isBlank(secret)){
            return AjaxResult.error("公司Key不能为空");
        }

//        if (StringUtils.isBlank(paramJson.getString("yhm"))){
//            return AjaxResult.error("电子税局登录名不能为空");
//        }

        Companyservice companyservice = iCompanyserviceService.selectCompanyserviceByIdentity(identity);

        if (companyservice == null || !secret.equals(companyservice.getSecret())) {
            return AjaxResult.error(ErrorCode.COMPANY_NOT_FOUND);
        }

        return elephantInvoiceService.getRpaQrCode(paramJson,companyservice);

    }


    /**
     * 数电登录接口(前端使用)
     */
    @ApiImplicitParams({
            @ApiImplicitParam(name = "identity", value = "身份认证", dataType = "java.lang.String", example = "1130", required = true),
            @ApiImplicitParam(name = "secret", value = "身份认证Key", dataType = "java.lang.String", example = "LTEO+oOgWMsuQAOUglqXuQ==", required = true),
            @ApiImplicitParam(name = "params", value = "请求体", dataType = "java.lang.String", required = true)
    })
    @PostMapping("/getRpaAuthStatus")
    public Object getRpaAuthStatus(@RequestBody Map<String,Object> params) {

        JSONObject paramJson = new JSONObject(params);

        log.debug("getRpaAuthStatus param{}", paramJson.toJSONString());
        String identity = paramJson.getString("identity");
        String secret = paramJson.getString("secret");

        //参数校验
        if (StringUtils.isBlank(identity)){
            return AjaxResult.error("公司ID不能为空");
        }

        if (StringUtils.isBlank(secret)){
            return AjaxResult.error("公司Key不能为空");
        }

        if (StringUtils.isBlank(paramJson.getString("rzid"))){
            return AjaxResult.error("认证ID不能为空");
        }

        Companyservice companyservice = iCompanyserviceService.selectCompanyserviceByIdentity(identity);
        if (companyservice == null || !secret.equals(companyservice.getSecret())) {
            return AjaxResult.error(ErrorCode.COMPANY_NOT_FOUND);
        }

        return elephantInvoiceService.getRpaAuthStatus(paramJson,companyservice);

    }


    /**
     * 接收大象回推发票信息接口
     */
    @Validated
    @PostMapping("/receiveInvoiceFromElephant")
    public Object receiveInvoiceFromElephant(@RequestBody String invoiceParam){

        //判断参数是否为空
        if (StrUtil.isEmpty(invoiceParam)) {
            return JSONObject.toJSONString(buildDxhyInterfaceResponse("9999", "推送发票信息为空", null));
        }

        log.info("receiveInvoiceFromElephant invoiceParam:{}", invoiceParam);

        //获取主信息
        JSONObject jsonParam = JSONObject.parseObject(invoiceParam);

        String content = jsonParam.getString("content");
//        log.info("receiveInvoiceFromElephant content:{}", content);
        //判断主信息是否为空
        if (StrUtil.isEmpty(content)) {
            log.info("推送发票content信息为空");
            return JSONObject.toJSONString(buildDxhyInterfaceResponse("9999", "推送发票信息为空", null));
        }
        String decodeParam = Base64Util.decodeToString(content);

        List<DdfpzxxV5> ddfpzxxV5s = JSONArray.parseArray(decodeParam, DdfpzxxV5.class);

        List<ResponseBO> responseBOs = new ArrayList<>();
        for (DdfpzxxV5 ddfpzxxV5 : ddfpzxxV5s) {
            if (ObjectUtils.isEmpty(ddfpzxxV5.getDDFPXX())){
                log.info("推送发票主体信息为空");
                return JSONObject.toJSONString(buildDxhyInterfaceResponse("9999", "推送发票主体信息为空", null));
            }

            if (ObjectUtils.isEmpty(ddfpzxxV5.getDDMXXX())){
                log.info("推送发票明细信息为空");
                return JSONObject.toJSONString(buildDxhyInterfaceResponse("9999", "推送发票明细信息为空", null));
            }

            Companyservice companyservice = iCompanyserviceService.selectCompanyserviceByTaxcode(ddfpzxxV5.getDDFPXX().getXHFSBH());

            log.info("推送发票销方税号:{}", ddfpzxxV5.getDDFPXX().getXHFSBH());
            if (companyservice == null){
                log.info("未查询到销方税号对应企业");
                return JSONObject.toJSONString(buildDxhyInterfaceResponse("9999", "未查询到销方税号对应企业", null));
            }
            ResponseBO responseBO;
            try {
                AjaxResult ajaxResult = elephantInvoiceService.convertAndSaveInvoice(ddfpzxxV5,companyservice);
                if (ajaxResult.isSuccess()){
                    responseBO = buildResponseBO(ddfpzxxV5, "000000", "");
                }else {
                    responseBO = buildResponseBO(ddfpzxxV5, "9999", ajaxResult.getMsg());
                }


            }catch (Exception e){
                log.error("推送发票异常{}", e);
                responseBO = buildResponseBO(ddfpzxxV5, "9999", e.getMessage());
            }
            responseBOs.add(responseBO);

        }

        return JSONObject.toJSONString(buildDxhyInterfaceResponse("000000", "成功", Base64Util.encodeToString(JSONObject.toJSONString(responseBOs).getBytes(StandardCharsets.UTF_8))));
    }


    public static DxhyInterfaceResponse buildDxhyInterfaceResponse(String returnCode, String returnMessage,String content) {
        DxhyInterfaceResponse dxhyInterfaceResponse = new DxhyInterfaceResponse();
        dxhyInterfaceResponse.setContent(content);
        dxhyInterfaceResponse.setEncryptCode("0");
        dxhyInterfaceResponse.setEncryptCode("0");
        dxhyInterfaceResponse.setReturnCode(returnCode);
        dxhyInterfaceResponse.setReturnMessage(returnMessage);
        return dxhyInterfaceResponse;
    }

    public ResponseBO buildResponseBO(DdfpzxxV5 ddfpzxxV5,String ztdm,String ztxx) {
        ResponseBO responseBO = new ResponseBO();
        responseBO.setDDQQLSH(ddfpzxxV5.getDDFPXX().getDDQQLSH());
        responseBO.setNSRSBH(ddfpzxxV5.getDDFPXX().getXHFSBH());
        responseBO.setZTXX(ztxx);
        responseBO.setZTDM(ztdm);
        return responseBO;
    }


}
