package com.jianshui.invoice.task;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.jianshui.common.core.domain.entity.Companyservice;
import com.jianshui.common.core.redis.RedisCache;
import com.jianshui.common.utils.StringUtils;
import com.jianshui.common.utils.spring.SpringUtils;
import com.jianshui.common.utils.uuid.IdUtils;
import com.jianshui.framework.manager.AsyncManager;
import com.jianshui.invoice.domain.BillInfo;
import com.jianshui.invoice.domain.Invoice;
import com.jianshui.invoice.domain.InvoiceBack;
import com.jianshui.invoice.domain.dto.QueryBillInfoDTO;
import com.jianshui.invoice.mapper.InvoiceBackMapper;
import com.jianshui.invoice.mapper.InvoiceMapper;
import com.jianshui.invoice.service.IInvoiceApiService;
import com.jianshui.queue.utils.RedisQueueUtil;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import lombok.extern.slf4j.XSlf4j;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.Date;
import java.util.TimerTask;

/**
 * @Description 发票查询任务
 * @Author 巩权林
 * @Date 2022/4/16 00:21
 **/
@Slf4j
public class QueryInvoiceTask extends TimerTask {

    private BillInfo finalBillInfo;
    private Companyservice companyservice;
    private IInvoiceApiService invoiceApiService;


    private InvoiceBackMapper invoiceBackMapper;


    public QueryInvoiceTask(BillInfo billInfo, Companyservice companyservice, IInvoiceApiService invoiceApiService,InvoiceBackMapper invoiceBackMapper) {
        this.finalBillInfo = billInfo;
        this.companyservice = companyservice;
        this.invoiceApiService = invoiceApiService;
        this.invoiceBackMapper = invoiceBackMapper;
    }

    @SneakyThrows
    @Override
    public void run() {
        RedisCache redisCache = SpringUtils.getBean(RedisCache.class);
        InvoiceMapper invoiceMapper = SpringUtils.getBean(InvoiceMapper.class);
        Thread.sleep(10 * 1000); // 睡眠10秒再去查询
        log.info("【定时任务】【发票查询】开始查询发票：{}", JSONObject.toJSONString(this.finalBillInfo));
        QueryBillInfoDTO queryBillInfoDTO = new QueryBillInfoDTO();
        queryBillInfoDTO.setFpqqlsh(this.finalBillInfo.getSystemOrderno());
        invoiceApiService.queryInvoice(queryBillInfoDTO, this.companyservice);

        // 获取发票
        Invoice invoice = invoiceMapper.selectBySystemOrderNo(this.companyservice.getCompanyid(), this.finalBillInfo.getSystemOrderno());
        if (invoice == null || StringUtils.equalsAny(Integer.toString(invoice.getState()), "0", "20")) {
            // 查询下失败队列，是否有次数
            String keys = "invoice_query_times_" + this.companyservice.getCompanyid() + "_" + this.finalBillInfo.getSystemOrderno();
            Integer queryTimes = redisCache.getCacheObject(keys);
            // 查询重试次数超过20次，就不查了
            if (queryTimes != null && queryTimes > 20) {
                log.error("【定时任务】【发票查询】发票信息始终不存在，不再进行查询：{}", JSONObject.toJSONString(this.finalBillInfo));
                redisCache.deleteObject(keys);
                return;
            }
            if (queryTimes == null) {
                queryTimes = 0;
            }
            queryTimes += 1;
            redisCache.setCacheObject(keys, queryTimes);
            log.error("【定时任务】【发票查询】发票信息不存在或还在开票中，再次进入查询：{}", JSONObject.toJSONString(this.finalBillInfo));
            AsyncManager.me().execute(new QueryInvoiceTask(this.finalBillInfo, companyservice, this.invoiceApiService,null));
        } else if (invoice == null || StringUtils.equalsAny(Integer.toString(invoice.getState()), "2", "22")) {
            // 2开票成功  22开票失败
            // 再次推送
            pushCoustomInvoiceInfo(invoice);
        }


        RedisQueueUtil.build().setData(invoice).onQueue("invoice_add_callback_consumer").retry(0).dispatch();
    }

    /** 发票开具回调 */
    private void pushCoustomInvoiceInfo(Invoice invoice) {

        try {
            log.info("【重汽批量开票】进入发票批量开具回调队列：InvoiceId={},{}",invoice.getId(),invoiceBackMapper);
            String kpStatus = Integer.toString(invoice.getState()).equals("2") ? "0000":"9999";
            String kpMsg = Integer.toString(invoice.getState()).equals("2") ? "开票成功":"开票失败";

            QueryWrapper<InvoiceBack> invoiceBackQueryWrapper = new QueryWrapper<>();
            invoiceBackQueryWrapper.eq("system_orderno", invoice.getOutTradeOrderno());
            InvoiceBack invoiceBackVali = invoiceBackMapper.selectOne(invoiceBackQueryWrapper);

            log.info("【重汽批量开票】查询回调表信息：{}",JSONUtil.toJsonStr(invoiceBackVali));
            if (BeanUtil.isEmpty(invoiceBackVali)) {
                invoiceBackVali = new InvoiceBack();
                invoiceBackVali.setSystemOrderno(invoice.getOutTradeOrderno());
                invoiceBackVali.setCreateTime(new Date());

                invoiceBackVali.setId(IdUtils.randomUUID());
                invoiceBackVali.setIdentity(companyservice.getIdentity());
                invoiceBackVali.setBackUrl("");
                invoiceBackVali.setStatus("0");
                invoiceBackVali.setCreateTime(new Date());
                invoiceBackVali.setSystemOrderno(invoice.getFpqqlsh());
                invoiceBackVali.setBackMsg("");
                invoiceBackVali.setResultCode(kpStatus);
                invoiceBackVali.setResultMsg(kpMsg);
                invoiceBackVali.setUpdateTime(new Date());
                invoiceBackMapper.insertInvoiceBack(invoiceBackVali);

            } else {
                invoiceBackVali.setResultCode(kpStatus);
                invoiceBackVali.setResultMsg(kpMsg);
                invoiceBackVali.setStatus("0");
                invoiceBackVali.setUpdateTime(new Date());
                invoiceBackMapper.updateInvoiceBack(invoiceBackVali);
            }

            log.info("【重汽批量开票】发票回调队列开始回调，组装参数：回调参数={}，identity={}，invoiceId={}", invoiceBackVali.toString(),companyservice.getIdentity(),invoice.getId());
            invoiceApiService.callBackAisino(invoiceBackVali, "", companyservice.getIdentity(), invoice.getId());
            log.info("【重汽批量开票】 发票批量开具回调队列处理完成: InvoiceId={}",invoice.getId());
        } catch (Exception e) {
            log.info("【重汽批量开票】发票回调队列失败！invoiceId={},捕捉异常{}", invoice.getId(),e.toString());
        }

    }


}
